package com.aitrainer.api.controller

import com.aitrainer.api.model.ExercisePlan
import com.aitrainer.api.model.ExercisePlanDetail
import com.aitrainer.api.model.ExerciseType
import com.aitrainer.api.repository.ExercisePlanDetailRepository
import org.slf4j.LoggerFactory
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class ExercisePlanDetailController( private val exercisePlanDetailRepository: ExercisePlanDetailRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @PostMapping("/exercise_plan_detail")
    fun createNewExercisePlanDetail(@Valid @RequestBody exercisePlanDetail: ExercisePlanDetail): ResponseEntity<ExercisePlanDetail> {
        logger.info("Create new exercise plan: $exercisePlanDetail")
        return ResponseEntity.ok().body(exercisePlanDetailRepository.save(exercisePlanDetail))
    }

    @GetMapping("/exercise_plan_detail/{exercisePlanId}")
    fun getExercisePlanByExercisePlanId(@PathVariable(value = "exercisePlanId") exercisePlanId: Int): ResponseEntity<List<ExercisePlanDetail>> {
        val list: List<ExercisePlanDetail> = exercisePlanDetailRepository.findByExercisePlanId(exercisePlanId)
        logger.info("-- getExercisePlanByExercisePlanId id: $exercisePlanId -- $list")
        return ResponseEntity.ok().body(list)
    }

    @Secured
    @PostMapping("/exercise_plan_detail/{exercisePlanDetailId}")
    fun updateExercisePlanDetailById(@PathVariable(value = "exercisePlanDetailId") exercisePlanDetailId: Long,
                               @Valid @RequestBody exercisePlanDetail: ExercisePlanDetail,
                               @RequestHeader requestHeader: RequestHeader?
    ): ResponseEntity<ExercisePlanDetail> {
        val returnPlanDetail = exercisePlanDetailRepository.findById(exercisePlanDetailId).orElse(null)
                ?: return ResponseEntity.notFound().build()

        val updatedPlanDetail = returnPlanDetail.copy(
                exercisePlanId = exercisePlanDetail.exercisePlanId,
                exerciseTypeId = exercisePlanDetail.exerciseTypeId,
                serie = exercisePlanDetail.serie,
                repeats = exercisePlanDetail.repeats,
                weightEquation = exercisePlanDetail.weightEquation
        )
        updatedPlanDetail.exercisePlanDetailId = exercisePlanDetail.exercisePlanDetailId
        logger.info("-- updateExercisePlanDetailById id: $updatedPlanDetail ")
        return ResponseEntity.ok().body(exercisePlanDetailRepository.save(updatedPlanDetail))
    }

    @Secured
    @PostMapping("/exercise_plan_detail/delete/{exercisePlanDetailId}")
    fun deleteExercisePlanDetailById(@PathVariable(value = "exercisePlanDetailId") exercisePlanDetailId: Long)
            :ResponseEntity<HttpStatus> {
        val returnPlanDetail = exercisePlanDetailRepository.findById(exercisePlanDetailId).orElse(null)
                ?: return ResponseEntity.notFound().build()
        try {
            exercisePlanDetailRepository.delete(returnPlanDetail)
        } catch (e: IllegalArgumentException ) {
            return ResponseEntity.ok().body(HttpStatus.BAD_REQUEST)
        }

        return ResponseEntity.ok().body(HttpStatus.OK)
    }
}