package com.aitrainer.api.controller

import com.aallam.openai.api.BetaOpenAI
import com.aitrainer.api.model.OpenAI
import com.aitrainer.api.model.OpenAIChat
import com.aitrainer.api.openai.OpenAIService
import kotlinx.coroutines.*
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.web.bind.annotation.*

@RestController
@RequestMapping("/api")
class OpenAIController() {
    private val logger = LoggerFactory.getLogger(javaClass)

    @OptIn(DelicateCoroutinesApi::class)
    @PostMapping("/openai/completion")
    fun getOpenAIResponse(@RequestBody question: String, @Value("\${openai.key}") openaiKey: String) : String {
        var result: String
        val openAIService = OpenAIService(openaiKey, null, null)
        val deferred = GlobalScope.async {
            openAIService.completion(question)
        }
        runBlocking {
            result = deferred.await()
            //println("Result: $result" )
        }
        return result
    }

    @OptIn(DelicateCoroutinesApi::class)
    @PostMapping("/openai/completion_with_model")
    fun getOpenAIResponseWithModel(@RequestBody openai: OpenAI, @Value("\${openai.key}") openaiKey: String) : String {
        var result: String
        val openAIService = OpenAIService(openaiKey, openai.modelName, openai.temperature)
        val deferred = GlobalScope.async {
            openAIService.completion(openai.question)
        }
        runBlocking {
            result = deferred.await()
            //println("Result: $result" )
        }
        return result
    }

    @OptIn(BetaOpenAI::class, DelicateCoroutinesApi::class)
    @PostMapping("/openai/chat_completion")
    fun getOpenAIChatCompletion(@RequestBody openai: OpenAIChat, @Value("\${openai.key}") openaiKey: String, ) : String {
        var result: String
        val openAIService = OpenAIService(openaiKey, openai.modelName, openai.temperature)
        val deferred = GlobalScope.async {
            println(openai.messages)
            openAIService.chatCompletion(openai.messages)
        }
        runBlocking {
            result = deferred.await().toString()
        }
        return result
    }

    @OptIn(DelicateCoroutinesApi::class)
    @GetMapping("/openai/list_models")
    fun getOpenAIModels(@Value("\${openai.key}") openaiKey: String): MutableList<String> {
        var result: MutableList<String>
        val openAIService = OpenAIService(openaiKey,null, null)
        val deferred = GlobalScope.async {
            openAIService.getModels()
        }
        runBlocking {
            result = deferred.await()
            //println("Result: $result" )
        }
        return result
    }
}

