package com.aitrainer.api.controller

import com.aitrainer.api.model.*
import com.aitrainer.api.model.diet.DietSensitivity
import com.aitrainer.api.model.diet.Meal
import com.aitrainer.api.model.diet.Recipe
import com.aitrainer.api.model.diet.Store
import com.aitrainer.api.repository.*
import com.aitrainer.api.repository.diet.*
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import com.google.gson.GsonBuilder
import org.springframework.web.bind.annotation.CrossOrigin


@RestController
@RequestMapping("/api")
class PackageController(private val exerciseAbilityRepository: ExerciseAbilityRepository,
                        private val exerciseTypeRepository: ExerciseTypeRepository,
                        private val exerciseTreeRepository: ExerciseTreeRepository,
                        private val propertyRepository: PropertyRepository,
                        private val productRepository: ProductRepository,
                        private val exerciseDeviceRepository: ExerciseDeviceRepository,
                        private val exerciseTreeParentsRepository: ExerciseTreeParentsRepository,
                        private val exercisePlanTemplateRepository: ExercisePlanTemplateRepository,
                        private val evaluationRepository: EvaluationRepository,
                        private val sportRepository: SportRepository,
                        private val tutorialRepository: TutorialRepository,
                        private val descriptionRepository: DescriptionRepository,
                        private val faqRepository: FaqRepository,
                        private val trainingPlanRepository: TrainingPlanRepository,
                        private val splitTestsRepository: SplitTestsRepository,
                        private val trainingPlanDayRepository: TrainingPlanDayRepository,
                        private val appTextRepository: AppTextRepository,
                        private val trainingProgramRepository: TrainingProgramRepository,
                        private val membershipRepository: MembershipRepository,
                        private val storeRepository: StoreRepository,
                        private val recipeRepository: RecipeRepository,
                        private val mealRepository: MealRepository,
                        private val dietSensitivityRepository: DietSensitivityRepository
) {

    @GetMapping("/diet_package")
    @CrossOrigin(origins = ["http://localhost:48102"])
    fun getDietPackageData(): ResponseEntity<String> {
        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val listProperty:List<Property> = propertyRepository.getProperties()
        val listPropertyJson: String = gson.toJson(listProperty)

        val listMembership = membershipRepository.findAll()
        val listMembershipJson = gson.toJson(listMembership)

        val listStore = storeRepository.findAll()
        val listStoreJson = gson.toJson(listStore)

        val listRecipe = recipeRepository.findAll()
        val listRecipeJson = gson.toJson(listRecipe)

        val listRawMaterial = mealRepository.findAll()
        val listRawMaterialJson = gson.toJson(listRawMaterial)

        val listDietSensitivity = dietSensitivityRepository.findAll()
        val listDietSensitivityJson = gson.toJson(listDietSensitivity)

        val listAppText = appTextRepository.findAllWithTranslation()
        val listAppTextJson = gson.toJson(listAppText)

        val packageJson: String =
            getClassRecord(Property::class.simpleName, listPropertyJson) +
                    "|||" +  getClassRecord(Membership::class.simpleName, listMembershipJson) +
                    "|||" +  getClassRecord(Store::class.simpleName, listStoreJson) +
                    "|||" +  getClassRecord(Recipe::class.simpleName, listRecipeJson) +
                    "|||" +  getClassRecord(Meal::class.simpleName, listRawMaterialJson) +
                    "|||" +  getClassRecord(DietSensitivity::class.simpleName, listDietSensitivityJson) +
                    "|||" +  getClassRecord(AppText::class.simpleName, listAppTextJson)

        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    @GetMapping("/club_package")
    fun getClubPackageData(): ResponseEntity<String> {

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val listProperty:List<Property> = propertyRepository.getProperties()
        val listPropertyJson: String = gson.toJson(listProperty)

        val listExerciseType = exerciseTypeRepository.findAll()
        val listExerciseTypeJson = gson.toJson(listExerciseType)

        val listTrainingPlan = trainingPlanRepository.findAll()
        val listTrainingPlanJson = gson.toJson(listTrainingPlan)

        val listTrainingPlanDay = trainingPlanDayRepository.findAll()
        val listTrainingPlanDayJson = gson.toJson(listTrainingPlanDay)

        val listTrainingProgram = trainingProgramRepository.findAll()
        val listTrainingProgramJson = gson.toJson(listTrainingProgram)

        val listMembership = membershipRepository.findAll()
        val listMembershipJson = gson.toJson(listMembership)

        val packageJson: String =
                getClassRecord(Property::class.simpleName, listPropertyJson) +
                    "|||" +  getClassRecord(ExerciseType::class.simpleName, listExerciseTypeJson) +
                    "|||" +  getClassRecord(TrainingPlan::class.simpleName, listTrainingPlanJson) +
                    "|||" +  getClassRecord(TrainingPlanDay::class.simpleName, listTrainingPlanDayJson) +
                    "|||" +  getClassRecord(TrainingProgram::class.simpleName, listTrainingProgramJson) +
                    "|||" +  getClassRecord(Membership::class.simpleName, listMembershipJson)


        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    @GetMapping("/app_package")
    fun getPackageData(): ResponseEntity<String> {

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val listDevices: List<ExerciseDevice> = exerciseDeviceRepository.getExerciseDevices()
        val listDevicesJson: String = gson.toJson(listDevices)

        val listProducts = productRepository.findAll()
        val listProductsJson: String = gson.toJson(listProducts)

        val listProperty:List<Property> = propertyRepository.getProperties()
        val listPropertyJson: String = gson.toJson(listProperty)

        val listExerciseTree = exerciseTreeRepository.getActiveMenu()
        val listExerciseTreeJson: String = gson.toJson(listExerciseTree)

        val listExerciseType = exerciseTypeRepository.findAll()
        val listExerciseTypeJson = gson.toJson(listExerciseType)

        val listExerciseAbility = exerciseAbilityRepository.findAll()
        val listExerciseAbilityJson: String = gson.toJson(listExerciseAbility)

        val listExerciseTreeParents = exerciseTreeParentsRepository.findAll()
        val listExerciseTreeParentsJson: String = gson.toJson(listExerciseTreeParents)

        val listPlanTemplate = exercisePlanTemplateRepository.findAll()
        val listPlanTemplateJson: String = gson.toJson(listPlanTemplate)

        val listEvaluations = evaluationRepository.findAll()
        val listEvaluationJson: String = gson.toJson(listEvaluations)

        val listSports = sportRepository.getSports()
        val listSportsJson: String = gson.toJson(listSports)

        val listTutorials = tutorialRepository.findAll()
        val listTutorialJson: String = gson.toJson(listTutorials)

        val listDescriptions = descriptionRepository.findAll()
        val listDescriptionJson: String = gson.toJson(listDescriptions)

        val listFaq = faqRepository.findAll()
        val listFaqJson: String = gson.toJson(listFaq)

        val listTrainingPlan = trainingPlanRepository.findAll()
        val listTrainingPlanJson = gson.toJson(listTrainingPlan)

        val listSplitTests = splitTestsRepository.findAll()
        val listSplitTestsJson = gson.toJson(listSplitTests)

        val listTrainingPlanDay = trainingPlanDayRepository.findAll()
        val listTrainingPlanDayJson = gson.toJson(listTrainingPlanDay)

        val listAppText = appTextRepository.findAll()
        val listAppTextJson = gson.toJson(listAppText)

        val listTrainingProgram = trainingProgramRepository.findAll()
        val listTrainingProgramJson = gson.toJson(listTrainingProgram)

        val packageJson: String =
                    getClassRecord(ExerciseDevice::class.simpleName, listDevicesJson) +
           "|||" +  getClassRecord(Product::class.simpleName, listProductsJson) +
           "|||" +  getClassRecord(Property::class.simpleName, listPropertyJson) +
           "|||" +  getClassRecord(ExerciseTree::class.simpleName, listExerciseTreeJson) +
           "|||" +  getClassRecord(ExerciseType::class.simpleName, listExerciseTypeJson) +
           "|||" +  getClassRecord(ExerciseAbility::class.simpleName, listExerciseAbilityJson) +
           "|||" +  getClassRecord(ExerciseTreeParents::class.simpleName, listExerciseTreeParentsJson) +
           "|||" +  getClassRecord(ExercisePlanTemplate::class.simpleName, listPlanTemplateJson) +
           "|||" +  getClassRecord(Evaluation::class.simpleName, listEvaluationJson) +
           "|||" +  getClassRecord(Sport::class.simpleName, listSportsJson) +
           "|||" +  getClassRecord(Tutorial::class.simpleName, listTutorialJson) +
           "|||" +  getClassRecord(Description::class.simpleName, listDescriptionJson) +
           "|||" +  getClassRecord(Faq::class.simpleName, listFaqJson) +
           "|||" +  getClassRecord(TrainingPlan::class.simpleName, listTrainingPlanJson) +
           "|||" +  getClassRecord(SplitTests::class.simpleName, listSplitTestsJson) +
           "|||" +  getClassRecord(TrainingPlanDay::class.simpleName, listTrainingPlanDayJson) +
           "|||" +  getClassRecord(AppText::class.simpleName, listAppTextJson)  +
           "|||" +  getClassRecord(TrainingProgram::class.simpleName, listTrainingProgramJson)

        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    fun getClassRecord(className: String?, json: String): String {
        return "$className***$json"
    }
}