package com.aitrainer.api.test.openai

import com.aallam.openai.api.BetaOpenAI
import com.aallam.openai.api.chat.ChatMessage
import com.aallam.openai.api.chat.ChatRole
import com.aitrainer.api.model.OpenAI
import com.aitrainer.api.model.OpenAIChat
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class OpenAITest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


   /* @Test
    fun `get a question successfully`() {
        val question = "Who the f. is Alice?"

        mockMvc.perform(
            MockMvcRequestBuilders.get("/api/openai/completion")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(question)
        )
            .andExpect(MockMvcResultMatchers.status().isOk)
            //.andExpect(MockMvcResultMatchers.content().string("Sorry I dont know"))

    }*/

    @OptIn(BetaOpenAI::class)
    @Test
    fun `get a chat message completion`() {
        val systemMsg = ChatMessage(
            role = ChatRole.User,
            content = "Te a Diet4You applikáció asszisztense vagy. Add meg ennek az ételnek a kalória és tápanyagadatait: Big Mac. Hány gramm egy adag ebből? A válasz ez az objektum JSON alakított formája legyen: Meal [cal: double, ch: double, fat: double, protein: double, sugar: double, portion: double]"
        )
        val listMessages = listOf(systemMsg)
        val gson = Gson()
        val messages = gson.toJson(listMessages)
        println("****** json $messages ********* ")


        val openai = OpenAIChat(
            messages = messages,
            modelName = "gpt-3.5-turbo",
            temperature = 0.1
        )

        val json: String = toJson(openai)
        println(json)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/openai/chat_completion")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(json)
        )
        .andExpect(MockMvcResultMatchers.status().isOk)

    }

    @OptIn(BetaOpenAI::class)
    @Test
    fun `get a chat message completion no modelname`() {
        val systemMsg = ChatMessage(
            role = ChatRole.User,
            content = "Te a Diet4You applikáció asszisztense vagy. Add meg ennek az ételnek a kalória és tápanyagadatait: 'Sült sütőtök 100g'. A válasz ez az objektum JSON alakított formája legyen: Meal [cal: double, ch: double, fat: double, protein: double, sugar: double, portion: double]. A 'portion' paraméter ezt tartalmazza: hány gramm egy átlagos adag ebből az ételből? "
        )
        val listMessages = listOf(systemMsg)
        val gson = Gson()
        val messages = gson.toJson(listMessages)
        println("****** json $messages ********* ")


        val openai = OpenAIChat(
            messages = messages
        )

        val json: String = toJson(openai)
        println(json)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/openai/chat_completion")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(json)
        )
            .andExpect(MockMvcResultMatchers.status().isOk)

    }

    @Test
    fun `get a question successfully with model name`() {
        val question = "Who the f. is Alice? Who sing that song?"
        val openai = OpenAI(
            question = question,
            modelName = "text-ada-001",
            temperature = 0.5
        )
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/openai/completion_with_model")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(openai))
        )
            .andExpect(MockMvcResultMatchers.status().isOk)
        //.andExpect(MockMvcResultMatchers.content().string("Sorry I dont know"))

    }

    @Test
    fun `get models  successfully`() {
        mockMvc.perform(
        MockMvcRequestBuilders.get("/api/openai/list_models")
            .contentType(MediaType.APPLICATION_JSON)
            .header("Authorization", "Bearer $authToken")
        )
        .andExpect(MockMvcResultMatchers.status().isOk)
    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}